#!/usr/bin/perl -w

#
# dbfilealter.pm
# Copyright (C) 2008 by John Heidemann <johnh@isi.edu>
# $Id: 3fdb1eb892c25ff5ac2449a75506608300dc4101 $
#
# This program is distributed under terms of the GNU general
# public license, version 2.  See the file COPYING
# in $dblibdir for details.
#

package Jdb::Filter::dbfilealter;

=head1 NAME

dbfilealter - alter the format of a JDB file, changing the row/column seperator

=head1 SYNOPSIS

dbfilealter [-c] [-F x] [-R x] [column...]

=head1 DESCRIPTION

This program reformats a JDB file,
altering the row or column separator.
It verifies that this action does not violate the
file constraints (for example, if spaces appear in data and 
the new format has space as a separator),
and optionally corrects things.

=head1 OPTIONS

=over

=item B<-F> or B<--fs> or B<--fieldseparator> S

Specify the field (column) separator as C<S>.

=item B<-R> or B<--rs> or B<--rowseparator> S

Specify the row separator as C<S>.

=item B<-c> or B<--correct>

Correct any inconstenences caused by the new separators,
if possible.

=back

=for comment
begin_standard_jdb_options

This module also supports the standard jdb options:

=over

=item B<-d>

Enable debugging output.

=item B<-i> or B<--input> InputSource

Read from InputSource, typically a file name, or C<-> for standard input,
or (if in Perl) a IO::Handle, Jdb::IO or Jdb::BoundedQueue objects.

=item B<-o> or B<--output> OutputDestination

Write to OutputDestination, typically a file name, or C<-> for standard output,
or (if in Perl) a IO::Handle, Jdb::IO or Jdb::BoundedQueue objects.

=item B<--autorun> or B<--noautorun>

By default, programs process automatically,
but Jdb::Filter objects in Perl do not run until you invoke
the run() method.
The C<--(no)autorun> option controls that behavior within Perl.

=item B<--help>

Show help.

=item B<--man>

Show full manual.

=back

=for comment
end_standard_jdb_options


=head1 SAMPLE USAGE

=head2 Input:

    #h name id test1
    a 1 80
    b 2 70
    c 3 65

=head2 Command:

    cat data.jdb | dbfilealter -F S

=head2 Output:

    #h -FS name id test1
    a  1  80
    b  2  70
    c  3  65
    #  | dbfilealter -F S

=head2 Command 2:

    cat data.jdb | dbfilealter -R C

=head2 Output:

    #L name id test1
    name: a
    id: 1
    test1: 80
    
    name: b
    id: 2
    test1: 70
    
    name: c
    id: 3
    test1: 65
    
    #   | dbfilealter -R C

=head1 SEE ALSO

L<Jdb>,
L<dbcoldefine>.


=head1 CLASS FUNCTIONS

=cut

@ISA = qw(Jdb::Filter);
($VERSION) = 2.0;

use strict;
use Pod::Usage;
use Carp;

use Jdb::Filter;
use Jdb::IO::Writer;


=head2 new

    $filter = new Jdb::Filter::dbfilealter(@arguments);

Create a new dbfilealter object, taking command-line arguments.

=cut

sub new ($@) {
    my $class = shift @_;
    my $self = $class->SUPER::new(@_);
    bless $self, $class;
    $self->set_defaults;
    $self->parse_options(@_);
    $self->SUPER::post_new();
    return $self;
}


=head2 set_defaults

    $filter->set_defaults();

Internal: set up defaults.

=cut

sub set_defaults ($) {
    my($self) = @_;
    $self->SUPER::set_defaults();
    $self->{_fscode} = undef;
    $self->{_rscode} = undef;
    $self->{_correct} = undef;
}

=head2 parse_options

    $filter->parse_options(@ARGV);

Internal: parse command-line arguments.

=cut

sub parse_options ($@) {
    my $self = shift @_;

    my(@argv) = @_;
    $self->get_options(
	\@argv,
 	'help|?' => sub { pod2usage(1); },
	'man' => sub { pod2usage(-verbose => 2); },
	'autorun!' => \$self->{_autorun},
	'close!' => \$self->{_close},
	'c|correct!' => \$self->{_correct},
	'd|debug+' => \$self->{_debug},
	'F|fs|cs|fieldseperator|columnseperator=s' => \$self->{_fscode},
	'i|input=s' => sub { $self->parse_io_option('input', @_); },
	'log!' => \$self->{_logprog},
	'o|output=s' => sub { $self->parse_io_option('output', @_); },
	'R|rs|rowseperator=s' => \$self->{_rscode},
	) or pod2usage(2);
    push (@{$self->{_cols}}, @argv);
}

=head2 setup

    $filter->setup();

Internal: setup, parse headers.

=cut

sub setup ($) {
    my($self) = @_;

    $self->finish_io_option('input', -comment_handler => $self->create_pass_comments_sub);

    # all the hard work is on the next line where we force the right codes
    my @out_args = ();
    push (@out_args, -fscode => $self->{_fscode})
	if (defined($self->{_fscode}));
    push (@out_args, -rscode => $self->{_rscode})
	if (defined($self->{_rscode}));
    $self->finish_io_option('output', -clone => $self->{_in}, @out_args);
}

=head2 run

    $filter->run();

Internal: run over each rows.

=cut
sub run ($) {
    my($self) = @_;
    # can't get any easier than this
    my $read_fastpath_sub = $self->{_in}->fastpath_sub();
    my $write_fastpath_sub = $self->{_out}->fastpath_sub();
    my $fref;
    while ($fref = &$read_fastpath_sub()) {
	# xxx: should correct errors
	&$write_fastpath_sub($fref);
    };
}

=head1 AUTHOR and COPYRIGHT

Copyright (C) 2008 by John Heidemann <johnh@isi.edu>

This program is distributed under terms of the GNU general
public license, version 2.  See the file COPYING
with the distribution for details.

=cut

1;
